#---------------------------------------------------------------
# File Output
#----------------------------------------------------------------

#
# Configuration

set file(name) "scanout-#.pnm"
set file(dir) [pwd]
set file(viewer) "xv -geometry +0+0"

set file(typeList) {
    { RAW  pnm  }
    { BMP  bmp  }
    { EPS  eps  }
    { GIF  gif  }
    { JPEG jpg  }
    { PCD  pcd  }
    { PCL  pcl  }
    { PCX  pcx  }
    { PDF  pdf  }
    { PNG  png  }
    { PS   ps   }
    { TIFF tiff }
    { XBM  xbm  }
    { XPM  xpm  }
}
set file(type) RAW
set file(imageonly) 1

foreach i $file(typeList) { set file(surfix_[lindex $i 0]) [lindex $i 1] }
set file(surfixList) "\\.pgm$|\\.ppm$"
foreach i $file(typeList) { append file(surfixList) "|\\.[lindex $i 1]$" }

#
# window_File and make_File

proc window_File { wn } { global file w l
    label $wn.ti -text "TkScan File Manager" -width 24 -pady 4 -relief ridge
    canvas $wn.sep -width 0 -height 5 -relief flat
    pack  $wn.ti $wn.sep -side top

    frame $wn.file -bd 1 -relief groove
    frame $wn.file.ft -bd 0
    pack  $wn.file.ft -side top -fill x
    label $wn.file.ft.l -text "File Type" -width 9 -anchor w
    menubutton $wn.file.ft.mb -menu $wn.file.ft.mb.m \
                              -width 8 -fg $l(c1) -relief ridge
    pack $wn.file.ft.l $wn.file.ft.mb -side left
    set w(filetype) $wn.file.ft.mb
    menu $wn.file.ft.mb.m -tearoff false
    foreach i $file(typeList) { set type [lindex $i 0]
         $wn.file.ft.mb.m add command -label $type -command "setFiletype $type"
    }
    frame $wn.file.fn -bd 0
    pack  $wn.file.fn -side top -fill x
    label $wn.file.fn.l -text "File Name" -width 9 -anchor w
    entry $wn.file.fn.e -textvariable file(name) \
                        -width 21 -fg $l(c1) -font $l(f1)
    pack  $wn.file.fn.l $wn.file.fn.e -side left

    frame $wn.dir -bd 1 -relief groove
    frame $wn.dir.l
    label $wn.dir.l.l -text Directory -width 9 -anchor w
    checkbutton $wn.dir.l.c1 -text "Image files only" \
                             -variable file(imageonly) -anchor w
    pack $wn.dir.l.l $wn.dir.l.c1 -side left -fill x
    entry $wn.dir.e -textvariable file(dir) \
                    -width 30 -relief sunken -font $l(f1) -fg $l(c1)
    frame $wn.dir.f
    listbox $wn.dir.f.b -width 28 -height 9 -relief sunken -font $l(f1) \
                        -yscroll "$wn.dir.f.s set"
    set w(dirlist) $wn.dir.f.b
    scrollbar $wn.dir.f.s -command "$wn.dir.f.b yview"
    pack $wn.dir.f.b -side left  -fill both -expand yes
    pack $wn.dir.f.s -side right -fill y
    frame $wn.dir.b
    button $wn.dir.b.1 -text view   -width 4 -pady 0 -relief ridge -command {
        if {[$w(dirlist) curselection] != {}} {
            set f [lindex [selection get] 1]
            if [file isdirectory $f] {
                cd $f
                set file(dir) [pwd]
                update_dir
            } else {
                exec sh -c "$file(viewer) $f" &
            }
        }
    }
    button $wn.dir.b.2 -text remove -width 4 -pady 0 -relief ridge -command {
        if {[$w(dirlist) curselection] != {}} {
            set f [lindex [selection get] 1]
            if [file isdirectory $f] {
            } else {
                exec sh -c "rm -f $f" &
                update_dir
            }
        }
    }
    button $wn.dir.b.3 -text mkdir  -width 4 -pady 0 -relief ridge -command {
        if ![file exists $file(dir)] {
            exec mkdir $file(dir)
            cd $file(dir)
            update_dir
        }
    }
    button $wn.dir.b.4 -text update -width 4 -pady 0 -relief ridge -command {
        update_dir
    }
    pack $wn.dir.b.1 $wn.dir.b.2 $wn.dir.b.3 $wn.dir.b.4 \
         -side left -fill x -expand yes
    pack $wn.dir.l $wn.dir.e $wn.dir.f $wn.dir.b -side top -fill x
    update_dir
    bind $wn.dir.f.b <Double-Button-1> {
        if {[$w(dirlist) curselection] != {}} {
            set f [lindex [selection get] 1]
            if [file isdirectory $f] {
                cd $f
                set file(dir) [pwd]
                update_dir
            } else {
                exec sh -c "$file(viewer) $f" &
            }
        }
    }
    bind $wn.dir.e <KeyPress-Return> { cd $file(dir) ; update_dir }

    pack $wn.file $wn.dir -side top -expand yes -fill x

    setcur Default
    setFiletype $file(type)
}

proc make_File { prefix i } { global file s c
    set scanned "$c(tkscandir)/$prefix-$i"
    # Output file name: replace # to $i
    set p [string last # $file(name)]
    if {$p >= 0} {
        set outfile [string range $file(name) 0 [expr $p - 1]]$i[string \
                               range $file(name) [expr $p + 1] end]
    } else {
        set outfile $file(name)
    }
    # Run ImageMagic's convert
    if {$s(gamma_support)==1} {
        set gamma "-gamma $s(gamma_red),$s(gamma_green),$s(gamma_blue)"
    } else {
        set gamma ""
    }
    exec sh -c \
        "convert $gamma $scanned.pnm $scanned-x.$file(surfix) ; \
         rm -f $scanned.pnm ; \
         mv $scanned-x.$file(surfix) $file(dir)/$outfile" &
    stat_text "Scanning $s(curscan) / $s(nrect) ... $outfile"
    update
    check_outfile $file(dir)/$outfile
}

#
# Subroutines

proc setFiletype { type } { global file w
    set file(type) $type
    $w(filetype) configure -text $type
    set file(surfix) $file(surfix_$type)
    set p [string last . $file(name)]
    if {$p > 0} {
        set file(name) [string range $file(name) 0 $p]$file(surfix)
    }
}

proc update_dir {} { global file w
    $w(dirlist) delete 0 end
    set filelist [exec ls -a $file(dir)]
    foreach i $filelist {
        if {[file isdirectory $file(dir)/$i] && $i!="." && !$file(imageonly)} {
            $w(dirlist) insert end [format " %9s  %-17s " "Directory" $i]
        }
    }
    foreach i $filelist {
        if { [file isfile $file(dir)/$i] && \
             ([regexp -nocase $file(surfixList) $i] || !$file(imageonly)) } {
            set size [expr [file size $file(dir)/$i] / 1024.0]
            $w(dirlist) insert end [format " %6.0f  %-20s " $size $i]
        }
    }
}

proc check_outfile { filename } {
    if [file exists $filename] {
        update_dir
    } else {
        after 1000 check_outfile $filename
    }
}

